#include <gurobi_c++.h>
using namespace std;

// Production planning of products for the next week.

int main(int argc, char *argv[]) {
  // Parameters.
  int numProducts = 7;
  int profits[numProducts] =         {10, 20, 20, 30, 15, 18, 10};
  int productionTimes[numProducts] = { 1,  5,  6,  3,  2,  1,  3};
  int availableProductionTime = 720;

  GRBEnv env;
  GRBModel model(env);

  // For each product, create a variable denoting the produced quantity of the product.
  // We will index the products from 0.
  GRBVar quantities[numProducts];
  for (int i = 0; i < numProducts; i++) {
    quantities[i] = model.addVar(0, GRB_INFINITY, 0.0, GRB_INTEGER);
  }

  model.update();

  // Constraint: Do not violate the available production time.
  {
    GRBLinExpr sumProductionTime = 0;
    for (int i = 0; i < numProducts; i++) {
      sumProductionTime += quantities[i] * productionTimes[i];
    }
    model.addConstr(sumProductionTime <= availableProductionTime);
  }

  // Constraint: do not produce more than 6 units of product 5.
  model.addConstr(quantities[5] <= 6);

  // Constraint: if no unit of product 3 is produced, product 2 also cannot be produced.
  {
    // This constraint requires the Big M trick. For this, we need to find out, what the Big M is so that
    // production of product 2 is not limited.
    double bigM = availableProductionTime / (1.0 * productionTimes[2]);
    model.addConstr(bigM * quantities[3] >= quantities[2]);
  }

  // Constraint: the ratio between produced quantities of products 5 and 0 is 1:3 or no units of
  // these two products are produced.
  model.addConstr(3 * quantities[5] == quantities[0]);

  // Objective: maximize profit.
  {
    GRBLinExpr objfn = 0;
    for (int i = 0; i < numProducts; i++) {
      objfn += quantities[i] * profits[i];
    }
    model.setObjective(objfn, GRB_MAXIMIZE);
  }

  model.optimize();

  if (model.get(GRB_IntAttr_Status) == GRB_OPTIMAL) {
    cout << "Optimal objective value: " << model.get(GRB_DoubleAttr_ObjVal) << endl;
    for (int i = 0; i < numProducts; i++) {
      cout << "Quantity of product " << i << ": " << quantities[i].get(GRB_DoubleAttr_X) << endl;
    }
  }

  return 0;
}
